#include <iostream>

#include "Scene.h"
#include "Particle.h"
#include "Cloth.h"
#include "Car.h"
#include "Shape.h"
#include "Program.h"


#include "Collider.h"
#include "Plane.h"
#include "Sphere.h"
#include "Box.h"

using namespace std;
using namespace Eigen;

Scene::Scene() :
	t(0.0),
	h(1e-2),
	grav(0.0, 0.0, 0.0)
{
}

Scene::~Scene()
{
}

void Scene::load(const string &RESOURCE_DIR)
{
	// Units: meters, kilograms, seconds
	h = 5e-3;//1.2e-2;
	
	grav << 0.0, -9.8, 0.0;
	
	int rows = 30;
	int cols = 30;
	double mass = 0.1;
	double stiffness = 20;//8;//1e1;
	Vector3d x00(-0.25, 0.5, 0.0);
	Vector3d x01(0.25, 0.5, 0.0);
	Vector3d x10(-0.25, 0.5, -0.5);
	Vector3d x11(0.25, 0.5, -0.5);
	// cloth = make_shared<Cloth>(rows, cols, x00, x01, x10, x11, mass, stiffness);

	int n = 5; // number of particles per side of the box - increase if sim running too fast, decrease if too slow
	cars.push_back(make_shared<Car>(n, n, n, Vector3d(0, 1, 0), 1, 2, 0.5, mass, stiffness));
	// cars.push_back(make_shared<Car>(6, 6, 6, Vector3d(1, 1, 0), 1, 2, 0.5, mass, stiffness));
	// cars.push_back(make_shared<Car>(2, 2, 2, Vector3d(-2, 1, 0), 1, 2, 0.5, mass, stiffness));
	
	for(auto car : cars)
		car->load(RESOURCE_DIR);
	
	sphereShape = make_shared<Shape>();
	sphereShape->loadMesh(RESOURCE_DIR + "sphere2.obj");
	
	auto sphere = make_shared<Particle>(sphereShape);
	// spheres.push_back(sphere);
	sphere->r = 0.1;
	sphere->x = Vector3d(0.0, 0.2, 0.0);

	shared_ptr<Plane> floor = make_shared<Plane>();
	floor->center = Vector3d(0, 0, 0);
    floor->normal = Vector3d(0, 1, 0);
    floor->xSize = -1;
	colliders.push_back(floor);

	colliders.push_back(make_shared<Box>(Vector3d(0, 0, -10), Vector3d(20, 0, 0), Vector3d(10, 1, 10)));
	colliders.push_back(make_shared<Box>(Vector3d(0, 7, -22), Vector3d(50, 0, 0), Vector3d(5, 1, 5)));
	colliders.push_back(make_shared<Sphere>(Vector3d(8, -2.5, 0), 3));
	// colliders.push_back(make_shared<Box>(Vector3d(0, 7, -22), Vector3d(40, 0, 0), Vector3d(2, 1, 8)));
	// colliders.push_back(make_shared<Box>(Vector3d(-8, 7, -22), Vector3d(50, 0, 0), Vector3d(2, 1, 5)));
	// colliders.push_back(make_shared<Box>(Vector3d(8, 7, -22), Vector3d(50, 0, 0), Vector3d(2, 1, 5)));

	for(auto collider : colliders) {
		collider->update();
		collider->load(RESOURCE_DIR);
	}
}

void Scene::init()
{
	sphereShape->init();
	// cloth->init();
	for(auto car : cars)
		car->init();
	for(auto collider : colliders)
		collider->init();
}

void Scene::tare()
{
	for(int i = 0; i < (int)spheres.size(); ++i) {
		spheres[i]->tare();
	}
	// cloth->tare();
	for(auto car : cars)
		car->tare();
}

void Scene::reset()
{
	t = 0.0;
	for(int i = 0; i < (int)spheres.size(); ++i) {
		spheres[i]->reset();
	}
	// cloth->reset();
	for(auto car : cars)
		car->reset();
}

void Scene::step(std::set<int> &pressedKeys, bool* keyToggles)
{
	t += h;
	
	// Move the sphere
	if(!spheres.empty()) {
		auto s = spheres.front();
		Vector3d x0 = s->x;
		s->x(2) = 0.5 * sin(0.5*t);
	}
	
	// Simulate the cloth
	// cloth->step(h, grav, spheres);
	for(auto car : cars)
		car->step(h, grav, spheres, pressedKeys, keyToggles, colliders);
}

void Scene::draw(shared_ptr<MatrixStack> MV, const shared_ptr<Program> prog) const
{
	glUniform3fv(prog->getUniform("kdFront"), 1, Vector3f(1.0, 1.0, 1.0).data());
	glUniform1f(prog->getUniform("alpha"), 1);
	for(int i = 0; i < (int)spheres.size(); ++i) {
		spheres[i]->draw(MV, prog);
	}
	// cloth->draw(MV, prog);
	for(auto car : cars)
		car->draw(MV, prog);
	for(auto collider : colliders)
		collider->draw(MV, prog);
}
